const Message = require('primea-message')
const Pipe = require('buffer-pipe')
const secp265k1 = require('secp265k1')
const leb128 = require('leb128').unsigned
const nacl = require('tweetnacl')

module.exports = class DfinityTx extends Message {
  serialize (inculdeSig = this.signature.length !== 0) {
    return Buffer.concat([
      leb128.encode(this.version),
      this.to,
      leb128.encode(this.caps),
      leb128.encode(this.ticks),
      leb128.encode(this.ticksPrice),
      leb128.encode(this.nonce),
      leb128.encode(this.data.length),
      this.data,
      this.publicKey
    ])
  }

  sign (secretKey) {
    const keyPair = nacl.sign.keyPair.fromSeed(secretKey)
    this.publicKey = keyPair.publicKey
    const serialized = this.serialize(false)
    const sig = nacl.sign.detached(serialized, keyPair.secretKey)
    return Buffer.concat([serialized, sig])
  }

  static validate (serialized) {
    const pubKey = serialized.slice(-96, -64)
    const sig = serialized.slice(-64)
    return nacl.sign.detached.verify(serialized.slice(0, -64), sig, pubKey)
  }

  static deserialize (raw) {
    const p = new Pipe(raw)
    const json = {
      version: leb128.read(p),
      to: p.read(20),
      caps: leb128.read(p),
      ticks: leb128.read(p),
      ticksPrice: leb128.read(p),
      nonce: leb128.read(p),
      height: leb128.read(p),
      data: p.read(leb128.read(p)),
      publicKey: p.read(32),
      siganture: p.buffer
    }
    return new DfinityEd25519Tx(json)
  }

  static get defaults () {
    return {
      version: 0,
      to: new Uint8Array(20),
      caps: 0,
      ticks: 0,
      ticksPrice: 0,
      nonce: 0,
      height: 0,
      data: new Uint8Array([]),
      publicKey: new Uint8Array(32),
      siganture: new Uint8Array([])
    }
  }
}
